#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Copyright (c) 2013, Richard Perlman All rights reserved.
# with special thanks to philippelt@users.sourceforge.net
# Public domain source code
#
# Version 0.6.0, January 2016, Jeremy Rutherford
#  - Added support for Netatmo Wind Gauge
#  - Added support for OAuth Token based authentication

################################################################################
# Imports
################################################################################
from berkinet import setLogLevel, versionCheck
import lnetatmo
import socket
import __builtin__
import time
import sys
import indigo  # Not necessary, but removes lots of lint errors

################################################################################
# Globals
################################################################################


class Plugin(indigo.PluginBase):
    ########################################
    # Class properties
    ########################################
    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
        indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)

        (self.logLevel, self.debug) = setLogLevel(self, pluginDisplayName, pluginPrefs.get("showDebugInfo1", "1"))

        self.userId = pluginPrefs.get("userId", "None")
        self.userPass = pluginPrefs.get("userPass", "None")
        self.clientId = pluginPrefs.get("userClient", "None")
        self.clientSecret = pluginPrefs.get("userSecret", "None")
        self.netatmoUnits = pluginPrefs.get("netatmoUnits", "None")
        self.netatmoFreq = int(pluginPrefs.get("netatmoFreq", 5))
        self.netatmoTtl = int(pluginPrefs.get("netatmoTtl", 60)) * 60
        self.netatmoTimeout = int(pluginPrefs.get("netatmoTimeout", 100))
        self.stopThread = False
        self.runNetatmo = True
        self.timerDict = {}
        __builtin__.pluginDisplayName = pluginDisplayName
        __builtin__.logLevel = self.logLevel

    ########################################
    def __del__(self):
        indigo.PluginBase.__del__(self)

    ########################################
    def startup(self):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct startup' ,  type=self.pluginDisplayName + ' Debug', isError=False)
        versionCheck(self, self.pluginDisplayName, self.logLevel, self.pluginId, self.pluginVersion)

    def deviceStartComm(self, dev):
        if dev.enabled and dev.configured:
            dev.stateListOrDisplayStateIdChanged()


  ########################################
    def readNetatmo(self):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct readNetatmo' ,  type=self.pluginDisplayName + ' Debug', isError=False)

        if self.userId == "None":
            self.errorLog("Plugin config is missing user credentials.")
            return

        authorization = lnetatmo.ClientAuth(username = self.userId, password = self.userPass, clientId = self.clientId, clientSecret = self.clientSecret)

        if authorization == 'error':
            indigo.server.log(u'Server error. Will retry in %s seconds' % (self.netatmoFreq*60),  type=self.pluginDisplayName, isError=True)
            return
        else:
            prId = "com.berkinet.Netatmo"
            netatmoDict = {}

            for netatmo in indigo.devices.iter(prId):
                if netatmo.configured:
                    module = netatmo.pluginProps['moduleName']
                    station = netatmo.pluginProps['stationName']

                    if not station in netatmoDict:
                        netatmoDict[station] = {}

                    if self.logLevel > 1: self.debugLog("Found Netatmo device '%s' (%s) for station:%s" % (netatmo.name, module, station))

                    netatmoDict[station][module] = netatmo
                    if self.logLevel > 1: self.debugLog("Added Netatmo device '%s' to update list" % (netatmo.name))

                else:
                    self.errorLog("Skipping Indigo device '%s'. Configuration not yet completed." % (netatmo.name))

            if self.logLevel > 2: self.debugLog("DONE: got here wth netatmoDict:%s" % netatmoDict)

            for station in netatmoDict:
                if self.logLevel > 1: self.debugLog("Queryng Netatmo station:%s" % (station))
                authorization = lnetatmo.ClientAuth(username = self.userId, password = self.userPass, clientId = self.clientId, clientSecret = self.clientSecret)
                if self.logLevel > 1: self.debugLog("Received Netatmo station authorization:%s" % (authorization))

                devList = lnetatmo.DeviceList(authorization)
                if self.logLevel > 1: self.debugLog("Received Netatmo station data:%s" % (devList))
                if self.logLevel > 1: indigo.server.log("TTL set to:%s seconds" % (self.netatmoTtl))
                for module, moduleData in devList.lastData(exclude=self.netatmoTtl , station=station).items():
                    if self.logLevel > 1: self.debugLog("Received Netatmo module:%s, Value=%s" % (module, moduleData))

                    # Before we get started, check to see if there is an Indigo device that matches the Netatmo device
                    try:
                        if netatmoDict[station][module].enabled:
                            for sensor, value in moduleData.items() :
                                if netatmoDict[station][module].enabled:
                                    if self.logLevel > 1 and sensor != "WindHistoric": self.debugLog("Starting Netatmo device update with: Module=%s, ID=%s, Sensor=%s, Value=%s" % (module, netatmoDict[station][module].id, sensor, value))

                                    if sensor == "When" or sensor == 'date_max_temp' or sensor == 'date_min_temp':
                                        value = time.strftime("%H:%M:%S",time.localtime(value))

                                    if self.netatmoUnits == "us":
                                        if sensor == "Temperature" or sensor == "max_temp" or sensor == "min_temp":
                                            value  = (value * 9/5) + 32
                                        if sensor == "Pressure":
                                            value = value / 33.86389
                                        if sensor == "Rain" or sensor == "sum_rain_24" or sensor == "sum_rain_1":
                                            value = value * 0.039370

                                    if sensor == "Co2":
                                        sensor = "CO2"
                                    if self.logLevel > 1 and sensor != "WindHistoric": self.debugLog("updating Netatmo device with: Module=%s, ID=%s, Sensor=%s, Value=%s" % (module, netatmoDict[station][module].id, sensor, value))

                                    # Setup for decimal place handling
                                    if 'decimalPlaces' in netatmoDict[station][module].pluginProps:
                                        decimalPlaces = int(netatmoDict[station][module].pluginProps['decimalPlaces'])
                                        if self.logLevel > 1: self.debugLog("using decimal place setting from props = %s" % (decimalPlaces))                      
                                    else:
                                        decimalPlaces = 1
                                        if self.logLevel > 1: self.debugLog("using default decimal place setting = %s" % (decimalPlaces))    

                                    if sensor == "Temperature" or sensor == "max_temp" or sensor == "min_temp" or sensor == "Pressure" or sensor == "Rain" or sensor == "sum_rain_24" or sensor == "sum_rain_1" or sensor == "WindStrength" or sensor == "WindAngle":
                                        if decimalPlaces == 0:
                                            value = '%0.0f' % value
                                        elif decimalPlaces == 1:
                                            value = '%0.1f' % value
                                    else:
                                        decimalPlaces = 0

                                    if sensor != "WindHistoric": netatmoDict[station][module].updateStateOnServer(key=sensor, value=value, decimalPlaces=decimalPlaces)

                                    if sensor == "battery_vp":
                                        levelval = value/70
                                        levelstr = u"%d%%" % (int(levelval))
                                        netatmoDict[station][module].updateStateOnServer("batteryLevel", levelval, uiValue=levelstr)
                        else:
                            if self.logLevel > 1: self.debugLog("Skiping un-enabled Netatmo device: Module=%s, ID=%s, Sensor=%s, Value=%s" % (module, netatmoDict[station][module].id, sensor, value))

                    except Exception, e:
                            self.errorLog("Unable to write data for Netatmo module '%s'. No matching Indigo device found." %module)
                            exc_type, exc_obj, exc_tb = sys.exc_info()
                            indigo.server.log("failed at: %s" % exc_tb.tb_lineno)


   ########################################
    def writeNetatmo(self, prDevice, theFunction, theVal):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct wrteNetatmo' ,  type=self.pluginDisplayName + ' Debug', isError=False)
        return

    ########################################
    def runConcurrentThread(self):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct runConcurrantThread' ,  type=self.pluginDisplayName + ' Debug', isError=False)

        try:
            while self.runNetatmo:
                self.readNetatmo()
                self.sleep(self.netatmoFreq*60)
        except self.StopThread:
            if self.logLevel > 0: indigo.server.log(u'Stopping Netatmo plugin',  type=self.pluginDisplayName, isError=False)
            pass
        except Exception, e:
            indigo.server.log(u'Plugin stopping. Reason = %s' % (e), type=self.pluginDisplayName, isError=True)

            if self.logLevel > 0: indigo.server.log(u'Stopping Netatmo device read loop',  type=self.pluginDisplayName, isError=False)
            pass

    ########################################
    # Netatmo Action callback
    ######################
    def actionControl(self, action, dev):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct actionControl' ,  type=self.pluginDisplayName + ' Debug', isError=False)
        if self.logLevel > 3: self.debugLog('Requested action: %s' % (action))

        if action.pluginTypeId == 'pollNetatmo':
            self.readNetatmo()

    ########################################
    # UI Validate, Close, and Actions defined in Actions.xml:
    ########################################
    def validateDeviceConfigUi(self, valuesDict, typeId, devId):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct validateDeviceConfigUi' ,  type=self.pluginDisplayName + ' Debug', isError=False)
        # We really should poll Netatmo to verify the existance of the device and module
        return (True, valuesDict)

   ########################################
    def closedPrefsConfigUi(self, valuesDict, userCancelled):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct closedPrefsConfigUi' ,  type=self.pluginDisplayName + ' Debug', isError=False)
        if userCancelled is False:
            (self.logLevel, self.debug) = setLogLevel(self, self.pluginDisplayName, valuesDict["showDebugInfo1"])
            self.userId = valuesDict["userId"]
            self.userPass = valuesDict["userPass"]
            self.clientId = valuesDict["userClient"]
            self.clientSecret = valuesDict["userSecret"]
            self.netatmoUnits = valuesDict["netatmoUnits"]
            self.netatmoFreq = int(valuesDict["netatmoFreq"])
            self.netatmoTimeout = int(valuesDict["netatmoTimeout"])

            socket.setdefaulttimeout(self.netatmoTimeout)

            if self.logLevel > 0: indigo.server.log("Plugin configuration re-loaded. Polling Netatmo's web server every %s minutes with a %s second timeout"
                                                    % (self.netatmoFreq, self.netatmoTimeout), type=self.pluginDisplayName)

            self.readNetatmo()

    ########################################
    def closedDeviceConfigUi(self, valuesDict, userCancelled, typeId, devId):
        if self.logLevel > 1: indigo.server.log(u'Entered: Funct closedDeviceConfigUi' ,  type=self.pluginDisplayName + ' Debug', isError=False)
        if userCancelled is False:
            if self.logLevel > 0: indigo.server.log("Netatmo device configuration changed. Polling Netatmo's web server for an update")

            self.readNetatmo()
